const express = require("express");
const body = require("body-parser");
const helmet = require("helmet");
const fs = require("fs-extra");
const liController = require("./ListingsController")
const cookieParser = require("cookie-parser");
const utils = require("./Utils");

const region_map = {

    "umbria": {
        description: "Il cuore verde dell'Italia",
        cities: ["Perugia", "Assisi", "Gubbio", "Todi", "Orvieto", "Spoleto", "Norcia", "Spello"]
    },
    "toscana": {
        description: "I borghi e le dolci colline",
        cities: ["Firenze", "Siena", "Pisa", "Lucca", "Montepulciano", "Volterra", "Montalcino", "San Gimignano", "Cortona"]
    },
    "emilia-romagna": {
        description: "Il buon cibo, le macchine sportive e il divertimento",
        cities: ["Bologna", "Parma", "Ferrara", "Maranello", "Rimini", "Riccione", "Ravenna"]
    }
}

var Router = function (port) {

    this.port = port;
    this.app = express();
    this.app.use(helmet({
        expectCt: true,
        frameguard: true,
        referrerPolicy: true
    }))
    this.app.enable('trust proxy', 1);
    this.app.disable('x-powered-by');
    this.app.use(cookieParser());
    this.app.use(body.json({ limit: '500mb' }));
    this.app.use(body.urlencoded({ limit: '500mb', extended: true, parameterLimit: 50000 }));
    this.app.set('views', __dirname + '/../views');
    this.app.set('view engine', 'twig');
    this.app.use('/assets', express.static(__dirname + '/../assets'))
    this.app.use('/uploads', express.static(__dirname + '/../data/uploads', {
        etag: true,
        expires: '1h'
    }))
    this.init = () => {

        return new Promise(

            async (resolve, reject) => {

                const languages = {
                    "it": JSON.parse(await fs.readFile('./lang/it.json')),
                    "pa": JSON.parse(await fs.readFile('./lang/pa.json')),
                    "fr": JSON.parse(await fs.readFile('./lang/fr.json'))
                }

                this.app.use((req, res, next) => {

                    let lang = /^[\w]{2}-[\w]{2},([\w]{2});.*/.exec(req.headers["accept-language"]);

                    if (lang.length > 0) {
                        if (languages.hasOwnProperty(lang[1])) {
                            res.lang = languages[lang[1]]
                            res.lang.lang_code = lang[1];
                        }
                        else {
                            res.lang = languages["it"]
                            res.lang.lang_code = "it";
                        }

                    } else { res.lang = languages["it"]; res.lang.lang_code = "it" }

                    next();

                })

                this.app.use('/dashboard', require('./routes/dashboard/'))
                this.app.use('/api', require('./routes/api'))

                this.app.get('/contact', (req, res) => {

                    res.render('contact', {

                        title: 'Contact us',
                        about_class: 'hidden',
                        lang: res.lang

                    })

                })

                this.app.get('/about', (req, res) => {

                    res.render('about', {

                        title: 'About us',
                        contact_class: 'hidden',
                        lang: res.lang

                    })

                })

                this.app.get('/', (req, res) => {

                    res.render('index', {

                        title: 'Homepage',
                        about_class: 'hidden',
                        contact_class: 'hidden',
                        lang: res.lang

                    })

                })

                this.app.get('/:region', async (req, res) => {

                    const query = await (utils.db.query("SELECT * FROM codes WHERE region_name = ? GROUP BY region_name", [req.params.region])),
                        key = req.params.region.toLowerCase();

                    if (query.length == 0) {

                        return res.render('404', {

                            title: 'Page not found',
                            about_class: 'hidden',
                            contact_class: 'hidden',
                            lang: res.lang

                        })

                    }

                    res.render('region', {

                        title: 'Results for ' + query[0].region_name,
                        region_name: query[0].region_name,
                        preamble: region_map[key],
                        lang: res.lang

                    })

                })

                this.app.get('/:region/:region_code/:item_code', async (req, res) => {

                    const query = (await (utils.db.query(`
            SELECT * FROM
            codes 
            INNER JOIN listings ON 
            listings.listing_province = codes.code
            WHERE codes.region_name = ? AND listings.listing_code_item = ? AND listings.listing_code_region = ?
          `, [req.params.region, req.params.item_code, req.params.region_code])))[0]

                    if (!query) {

                        return res.status(404).render('404', {

                            title: 'Page not found',
                            about_class: 'hidden',
                            contact_class: 'hidden',
                            lang: res.lang

                        })

                    }

                    let images = await utils.listings.getImages(query.listing_uuid)

                    let percentage = query.listing_service_tax / 100;

                    let current_index = parseInt(req.params.item_code);
                    let previous_index = utils.stringifyIndex(current_index - 1),
                        next_index = utils.stringifyIndex(current_index + 1);


                    let next = await (utils.db.query(`
          
            SELECT * FROM listings WHERE listing_code_region = ?
            AND listing_code_item = ?

          `, [req.params.region_code, next_index]))

                    let prev = await (utils.db.query(`
          
            SELECT * FROM listings WHERE listing_code_region = ?
            AND listing_code_item = ?

          `, [req.params.region_code, previous_index]))


                    res.render('listing', {

                        title: query.listing_title,
                        listing: query,
                        listing_price_locale: parseInt(query.listing_price).toLocaleString(),
                        listing_total_price_locale: parseFloat((query.listing_price * percentage) + query.listing_price).toLocaleString(),
                        listing_description_formatted: query.listing_description.replace(/\n/g, "<br>"),
                        images,
                        first_image: images[0],
                        next: next[0], prev: prev[0],
                        lang: res.lang

                    })

                })

                /*this.app.get('*', (req, res) => {
                 
                  return res.status(404).render('404', {
                 
                    title: 'Page not found',
                    about_class: 'hidden',
                    contact_class: 'hidden',
                 
                  })
                 
                })*/

                this.app.listen(this.port);

                resolve(this.port);

            }

        )

    }

}

module.exports = Router
